<#
# AddRemoveUserToGroup.ps1
#
# A program that demonstrates PowerShell integration with UEM REST API.
#
# This sample program demonstrates how to use REST authorization, add/remove user from groups.
# UEM REST API reference: https://help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
#>

$global:AuthorizationString = $null;
$global:UEMHostPortTenantGUIDBaseURL = $null;

<#
# <summary>
# Prompts users for information in order to generate authorization string to be used for authenticated REST route.
# </summary>
# <returns>Returns true when the setup is successful, and false otherwise.</returns>
#>
function Setup()
{
    $methodName = "Setup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));
    $provider = $null;

    <# Prompt the user for the UEM hostname, port, tentantGuid #>
    $UEMHostAndPort = Read-Host 'Enter UEM HostName:Port (E.g. UEM.host.name:18084) ';
    $TenantGuid = Read-Host 'Enter Tenant Id';

    $global:UEMHostPortTenantGUIDBaseURL = "https://"+$UEMHostAndPort + "/" + $TenantGuid+"/api/v1";

    do
    {
     try
        {
            [int]$authType = Read-Host "Authentication type: `n1) Active directory `n2) Local Account`n";

            if(($authType -eq 1) -or ($authType -eq 2))
            {
                $validNumber = $true;
            }
        }
        catch
        {
            $validNumber = $false;
        }
    }
    until
        ($validNumber)

    if($authType -eq 1)
    {
        $UserName = Read-Host 'Enter AD Username ';
        $Password = Read-Host 'Enter AD Password ';
        $Domain = Read-Host 'Enter Domain';
        $provider = "AD";
    }
    elseif($authType -eq 2)
    {
        $UserName = Read-Host 'Enter local Username ';
        $Password = Read-Host 'Enter local Password ';
        $provider = "LOCAL";
    }


    <# Convert Password to base64 #>
    $Bytes = [System.Text.Encoding]::UTF8.GetBytes($Password);
    $Base64Password = [Convert]::ToBase64String($Bytes);

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    <# Create body for REST call #>
    $body = @{}
    $body.add("provider",$provider);
    $body.add("username", $UserName);
    $body.add("password", $Base64Password);
    if($authType -eq 1)
    {
        $body.add("domain", $Domain);
    }

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/util/authorization";

    <# Invoke RESTMethod to call authorization route to generate the authorization headers #>
    try
    {
        $response = Invoke-RESTMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        $global:AuthorizationString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:AuthorizationString));
    Write-Host("Setup Complete...");
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $true;
}

<#
# <summary>
# Search for UEM user based on email address parameter
# </summary>
# <param>User email address to serach</param>
# <returns>Returns a list users based on the search query</returns>
#>
function getUEMUser($userEmailAddress)
 {

    $methodName = "getUEMUser()";
    Write-Host([string]::Format("Entering {0}...", $methodName));

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users?query=emailAddress="+$userEmailAddress;

    <# Invoke RESTMethod to call users API to obtain users in the system #>
    try
    {
        $response = Invoke-RESTMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
        handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $response;
 }

<#
# <summary>
# Retrieves group using the name search query
# </summary>
# <param>Group name to search</param>
# <returns>Returns the response object from the group search</returns>
#>
function searchForGroup($groupToSearch)
{
    $methodName = "searchForGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.groups-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/groups?query=name="+$groupToSearch;

    <# Invoke RESTMethod to call users API to obtain users in the system #>
    try
    {
        $response = Invoke-RESTMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
         handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $response;
}

<#
# <summaryAdd user to group call</summary>
# <param>User GUID obtained from get users call. Group GUID obtained from the search for groups call</param>
# <return>return true if the api call was a success</return>
#>
function addUserToGroup($userGUID, $groupGUID)
{
    $methodName = "addUserToGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));
    $returnValue = $false;
    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    <# Create body for REST call #>
    $body = @{};
    $innerBody = @(@{
    "guid" = $userGUID;
    });
    $body.Add("users",$innerBody);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/groups/"+ $groupGUID +"/users";

    <# Invoke RESTMethod to call users API to obtain users in the system #>
    try
    {
        $response = Invoke-RESTMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $true;
}


<#
# <summary>
# Remove User from group
# </summary>
# <param>User GUID obtained from get users call. Group GUID obtained from the search for group call</param>
# <return>return true if the api call was a success</return>
#>
function removeUserFromGroup($userGUID, $groupGUID)
{
    $methodName = "addUserToGroup()";
    Write-Host([string]::Format("Entering {0}...", $methodName));

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    <# Create body for REST call #>
    $body = @{};
    $innerBody = @(@{
    "guid" = $userGUID;
    });
    $body.Add("users",$innerBody);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/groups/"+ $groupGUID +"/users";

    <# Invoke RESTMethod to call users API to obtain users in the system #>
    try
    {
        $response = Invoke-RESTMethod -Method DELETE -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
    return $true;
}

<#
# <summary>
# Writes Entering method name
# </summary>
# <param>Method name</param>
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
# <summary>
# Writes Exiting method name.
# </summary>
# <param>Method name</param>
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
# <summary>Handle any errors that occur when performing the REST call.  On error the program will exit
# </summary>
# <param>The error</param>
# <return></return>
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

<#
# <summary>Check the length of user object that was passed in from search query.  Write to console if no results are returned, or if more then one are return.</summary>
# <param>Length, Type and Object </param>
# <return>return true if one user was found</return>
#>
function returnCheck ($length, $type, $object)
{
    $retunValue = $false;
    if($length -eq 1)
    {
        $retunValue = $true;
    }
    elseif($length -gt 1)
    {
        Write-Host([string]::Format("More then one {0} was found with name {1}", $type, $object));
    }
    elseif($length -eq 0)
    {
        Write-Host([string]::Format("No {0} was found with name {1}", $type, $object));
    }
    return $retunValue;
}

<#
# <summary>The Main Function</summary>
#>
function main
{
    $methodName = "Main()";
    Write-Host([string]::Format("Entering {0}...", $methodName));
    try
    {
        if (Setup)
        {
            do
            {
                try
                {
                [int]$actionType = Read-Host "1 ) Add user to group `n2 ) Remove user from group`n";
                 if($actionType -eq 1 -or $actionType -eq 2)
                    {
                        $validNumber = $true;
                    }
                }
                catch
                {
                $validNumber = $false;
                }
            }
            until
                ($validNumber)

                if($actionType -eq 1)
                    {
                     $userEmailAddress = Read-Host "Enter email address of user to add to group";
                     $userSearchResponse = getUEMUser($userEmailAddress);
                     if(returnCheck $userSearchResponse.users.length "user" $userEmailAddress)
                     {
                       Write-Host([string]::Format("User with email address {0} was found", $userSearchResponse.users.emailAddress));
                       $groupToSearch = Read-Host "Enter group name";
                       $groupSearchResponse = searchForGroup($groupToSearch);
                       if(returnCheck $groupSearchResponse.groups.length "group" $groupToSearch)
                       {
                        Write-Host([string]::Format("Group with name {0} was found", $groupSearchResponse.groups.name));

                            if(addUserToGroup $userSearchResponse.users.guid $groupSearchResponse.groups.guid)
                            {
                                Write-Host([string]::Format("User {0} has been added to the group", $userEmailAddress));
                            }
                       }
                     }
                }
                elseif($actionType -eq 2)
                {
                     $userEmailAddress = Read-Host "Enter email address of user to remove from group";
                     $userSearchResponse = getUEMUser($userEmailAddress);
                     if(returnCheck $userSearchResponse.users.length "user" $userEmailAddress)
                     {
                       Write-Host([string]::Format("User with email address {0} was found", $userSearchResponse.users.emailAddress));

                       $groupToSearch = Read-Host "Enter group name";
                       $groupSearchResponse = searchForGroup($groupToSearch);
                       if(returnCheck $groupSearchResponse.groups.length "group", $groupToSearch)
                       {
                        Write-Host([string]::Format("Group with name {0} was found", $groupSearchResponse.groups.name));

                            if(removeUserFromGroup $userSearchResponse.users.guid $groupSearchResponse.groups.guid)
                            {
                                Write-Host([string]::Format("User {0} has been removed to the group", $userEmailAddress));
                            }
                       }
                     }
                }
        }
    }
    catch
    {
        handleError $_;
    }
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
# <summary>
# Calls main function
# </summary>
#>
main;