<#
# GetUsersAndDisplayDetails.ps1
#
# A program that demonstrates PowerShell integration with UEM REST API.
#
# This sample program demonstrates how to use REST authorization, get users with paging and sorting along with user devices routes in order to display information
# about the user and devices.
# UEM REST API reference: https://help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
#>


$global:AuthorizationString = $null;
$global:UEMHostPortTenantGUIDBaseURL = $null;

<#
# <summary>
# Prompts users for information in order to generate authorization string to be used for authenticated REST route.
# </summary>
# <returns>Returns true when the setup is successful</returns>
#>
function Setup()
{
    $methodName = "Setup()";
    enterMethod($methodName);
    $provider = $null;

    <# Prompt the user for the UEM hostname, port, tentantGuid #>
    $UEMHostAndPort = Read-Host 'Enter UEM HostName:Port (E.g. UEM.host.name:18084) ';
    $TenantGuid = Read-Host 'Enter Tenant Id';

    $global:UEMHostPortTenantGUIDBaseURL = "https://"+$UEMHostAndPort + "/" + $TenantGuid+"/api/v1";

    do
    {
     try
        {
            [int]$authType = Read-Host "Authentication type: `n1) Active directory `n2) Local Account`n";
            $validNumber = $true;
        }
        catch
        {
            $validNumber = $false;
        }
    }
    until
        ($validNumber)

    if($authType -eq 1)
    {
        $UserName = Read-Host 'Enter AD Username ';
        $Password = Read-Host 'Enter AD Password ';
        $Domain = Read-Host 'Enter Domain';
        $provider = "AD";
    }
    elseif($authType -eq 2)
    {
        $UserName = Read-Host 'Enter local Username ';
        $Password = Read-Host 'Enter local Password ';
        $provider = "LOCAL";
    }


    <# Convert Password to base64 #>
    $Bytes = [System.Text.Encoding]::UTF8.GetBytes($Password);
    $Base64Password = [Convert]::ToBase64String($Bytes);

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    <# Create body for REST call #>
    $body = @{}
    $body.add("provider",$provider);
    $body.add("username", $UserName);
    $body.add("password", $Base64Password);
    if($authType -eq 1)
    {
        $body.add("domain", $Domain);
    }

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/util/authorization";

    <# Invoke RESTMethod to call authorization route to generate the authorization headers #>
    try
    {
        $response = Invoke-RESTMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        $global:AuthorizationString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:AuthorizationString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
# <summary>
# Prompts user to enter any key to continue to the next page of users.
# </summary>
# <param>Message to be passed in</param>
#>
function pause ($message)
{
    # Check if running Powershell ISE
    if ($psISE)
    {
        Add-Type -AssemblyName System.Windows.Forms
        [System.Windows.Forms.MessageBox]::Show("$message")
    }
    else
    {
        Write-Host "$message" -ForegroundColor Yellow
        $x = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
    }
}

<#
# <summary>
# Retrieves 10 users at a time in pages sorting by username DESC.  Loops through the users list and calls DisplayUserDetails() for each given user.
# </summary>
#>
function GetUsersAndDisplayUserData()
 {
    $methodName = "GetUsersAndDisplayUserData()";
    enterMethod($methodName);
    $userList = @{};
    $userList = $null;
    $pageCounter = 0;
    $offset = 0;
    $count = 0;

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    while($offset -le $count)
    {
        $offset = $pageCounter * 10;
        $request = $global:UEMHostPortTenantGUIDBaseURL+"/users?max=10&offset="+$offset+"&sortBy=username%20DESC";

        <# Invoke RESTMethod to call users API to obtain users in the system #>
        try
        {
            $response = Invoke-RESTMethod -Method GET -Headers $headers -Uri $request;
        }
        catch
        {
             handleError $_;
        }

        if($response -ne $null -AND $response.users.length -gt 0)
        {
            $pageCounter++;
            $count = $response.users.length + $count;
            foreach($user in $response.users)
            {
                DisplayUserDetails($user);
            }
        }
        else
        {
            Write-Host("No users were found");
            break;
        }

        pause "`nPress Any Key For Next Page of Users";
    }

    exitMethod($methodName);
 }

<#
# <summary> Displays details for each given user
# </summary
# <param name>User object obtained from get users call</param>
#>
function DisplayUserDetails($userDetails)
{
    $methodName = "DisplayUserDetails()";
    enterMethod($methodName);
    $response = $null;

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.userdetail-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$userDetails.guid+"/userDevices";

    <# Invoke RESTMethod to call userDevices #>
    try
    {
        $response = Invoke-RESTMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
        handleError $_;
    }

    Write-Host("--------------------------");
    Write-Host("User name: " +$userDetails.username);
    Write-host("Display name: "+$userDetails.displayName);
    Write-host("First name: " + $userDetails.firstName);
    Write-host("Last name: " + $userDetails.lastName);
    Write-host("Eamil address: " + $userDetails.emailAddress);
    Write-host("User Guid: " + $userDetails.guid);
    if($userDetails.directoryId -ne $null)
    {
        Write-host("Directory Id: " + $userDetails.directoryId);
    }

    if($response.userDevices.length -gt 0)
        {
            foreach($userDevice in $response.userDevices)
            {
                Write-Host("-- Device Details --");
                Write-Host("Enrollment Type: " + $userDevice.enrollmentType);
                Write-host("Activation Date: " + $userDevice.device.activationDate);
                Write-Host("Hardware Name: " + $userDevice.device.hardwareName);
                Write-Host("Hardware Model: " + $userDevice.device.hardwareModel);
                Write-Host("OS: "+ $userDevice.device.os);
                Write-Host("");
            }
        }
    else
    {
        Write-Host("No devices found for user");
    }

    exitMethod($methodName);
 }

<#
# <summary>
# Writes Entering method name
# </summary>
# <param>Method name</param>
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
# <summary>
# Writes Exiting method name.
# </summary>
# <param>Method name</param>
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
# <summary>
# Handle any errors that occur when performing the REST call.
# </summary>
# <param>The error</param>
#<return></return>
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

<#
# <summary>
# The Main Function.
# </summary>
#>
function main
{
    $methodName = "Main()";
    enterMethod($methodName);
    try
    {
        if (Setup)
        {
            Write-Host("Setup Complete...");
            GetUsersAndDisplayUserData;
        }
    }
    catch
    {
        handleError($_);
    }
    exitMethod($methodName);
}

<#
# <summary>
# Calls main function.
# </summary>
#>
main;